\DPR.XPL	11-Nov-2017	Loren Blaney	loren.blaney@gmail.com
\Double Print
\
\This saves paper by printing two pages side-by-side in landscape mode on
\ an HP LaserJet or InkJet printer. Pages are 80 characters by 60 lines.
\ Lines longer than 80 characters are wrapped to the next line (unlike
\ the HP default, which chops them off).
\
\    Usage: DPR filename.ext
\
\REVISIONS:
\ 30-Aug-1998: Original release
\ 12-Mar-2011: Wrap long lines properly when they contain tabs.
\ 11-Nov-2017: LF = newline, for Linux compatibility.

inc	c:\cxpl\codesi;

def	Tab=$09, LF=$0A, FF=$0C, CR=$0D, EOF=$1A, Esc=$1B;

\-------------------------------------------------------------------------------

proc	EscTxt(Cmd);	\Send escape command to printer
char	Cmd;
begin
ChOut(2, Esc);
Text(2, Cmd);
end;	\EscTxt

\-------------------------------------------------------------------------------

proc	OpenFile;	\Open the input file that was typed on the command line
int	CpuReg, Hand;
char	CmdTail($80);
begin
CpuReg:= GetReg;		\get input file name (get command tail from PSP)
Blit(CpuReg(11), $81, CpuReg(12), CmdTail, $7F);
Hand:= FOpen(CmdTail, 0);	\open it
FSet(Hand, ^I);
OpenI(3);
end;	\OpenFile

\-------------------------------------------------------------------------------

proc	PrintFile;	\Print the input file, two columns per page
int	Col, Row, Page, Ch;


	proc	NextPage;	\Next half-sheet of paper
	begin
	Page:= Page+1;
	if Page & 1 then
		begin
		ChOut(2, FF);		\eject sheet
		EscTxt("&a6L");		\set left margin
		end
	else	begin
		EscTxt("&a93L");	\set left margin
		EscTxt("&a0R");		\move to first row
		end;
	Row:= 0;
	ChOut(2, CR);			\move to first column
	Col:= 0;
	end;	\NextPage


begin	\PrintFile
Col:= 0;
Row:= 0;
Page:= 1;
loop	begin
	Ch:= ChIn(3);
	case Ch of
	  CR:	begin			\carriage return
		Col:= 0;
		ChOut(2, Ch);
		end;
	  LF:	begin			\line feed
		Col:= 0;		\Linux style
		ChOut(2, CR);
		Row:= Row+1;
		if Row >= 60 then NextPage else ChOut(2, LF);
		end;
	  FF:	begin			\form feed
		NextPage;
		end;
	  EOF:	quit			\end of file
	other	begin
		if Col >= 80 then
			begin		\wrap text to next line
			Col:= 0;
			ChOut(2, CR);
			Row:= Row+1;
			if Row >= 60 then NextPage else ChOut(2, LF);
			end;
		ChOut(2, Ch);
		if Ch = Tab then Col:= Col+8 & $FFF8
		else Col:= Col+1;
		end;
	end;	\loop
end;	\PrintFile

\-------------------------------------------------------------------------------

begin	\Main
OpenFile;

\Set up printer for two columns in landscape mode
EscTxt("E");		\reset
EscTxt("&l1O");		\landscape mode
EscTxt("(10U");		\PC-8 font with the IBM extended character set
EscTxt("(s16.67H");	\16.67 characters per inch
EscTxt("&l6C");		\VMI = 6/48" = 1/8" or 8 lines per inch
EscTxt("&l60F");	\text length = 60 lines (or bottom margin = 1/2")
\60"/8lpi = 7.5" of text. Plus half-inch margins top & bottom = 8.5".
EscTxt("&a6L");		\set left margin for first page

PrintFile;

\Eject any partial sheet and reset printer to its initial configuration
EscTxt("E");
end;	\Main
