\QPR.XPL	11-Nov-2017	Loren Blaney	loren.blaney@gmail.com
\Quad Print: This saves paper by printing four standard (80x60) pages on a
\ single sheet of paper. Pages are printed in two columns, each 80 characters by
\ 120 lines. Lines longer than 80 characters are wrapped to the next line. An HP
\ LaserJet or InkJet printer is required.
\
\    Usage: QPR filename.ext
\
\Compile with xjsb.bat

\REVISIONS:
\ 14-Aug-2007: Original release.
\ 11-Nov-2017: LF = newline, for Linux compatibility.

include	c:\cxpl\codesi;

def	LF=$0A, FF=$0C, CR=$0D, EOF=$1A, Esc=$1B;

\-------------------------------------------------------------------------------

proc	EscTxt(Cmd);	\Send escape command to printer
char	Cmd;
begin
ChOut(5, Esc);
Text(5, Cmd);
end;	\EscTxt

\-------------------------------------------------------------------------------

proc	OpenFile;	\Open the input file that was typed on the command line
int	CpuReg, Hand;
char	CmdTail($80);
begin
CpuReg:= GetReg;		\get input file name (get command tail from PSP)
Blit(CpuReg(11), $81, CpuReg(12), CmdTail, $7F);
Hand:= Fopen(CmdTail, 0);	\open it
FSet(Hand, ^I);
OpenI(3);
end;	\OpenFile

\-------------------------------------------------------------------------------

proc	PrintFile;	\Print the input file, two columns per page
int	Col, Row, Page, Ch;


	proc	NextQuarter;		\Next quarter sheet of paper
	begin
	Page:= Page + 1;
	case Page & 3 of
	  0:	begin			\print on left side of paper
		ChOut(5, FF);		\eject sheet
		EscTxt("&a10L");	\set left margin
		end;
	  1, 3:	EscTxt("&a60R");	\move to 60th row = 1st row in quarter
	  2:	begin			\print on right side of paper
		EscTxt("&a100L");	\set left margin
		EscTxt("&a0R");		\move to first row
		end
	other	[];

	Row:= 0;
	ChOut(5, CR);			\move to first column
	Col:= 0;
	end;	\NextQuarter


begin	\PrintFile
Col:= 0;
Row:= 0;
Page:= 0;
loop	begin
	Ch:= ChIn(3);
	case Ch of
	  CR:	begin			\carriage return
		Col:= 0;
		ChOut(5, Ch);
		end;
	  LF:	begin			\line feed
		Col:= 0;		\Linux style
		ChOut(5, CR);
		Row:= Row + 1;
		if Row >= 60 then NextQuarter else ChOut(5, LF);
		end;
	  FF:	begin			\form feed
		NextQuarter;
		end;
	  EOF:	quit			\end of file
	other	begin
		if Col >= 80 then
			begin		\wrap text to next line
			Col:= 0;
			ChOut(5, CR);
			Row:= Row + 1;
			if Row >= 60 then NextQuarter else ChOut(5, LF);
			end;
		ChOut(5, Ch);
		Col:= Col + 1;
		end;
	end;	\loop
end;	\PrintFile

\-------------------------------------------------------------------------------

begin	\Main
OpenFile;

\Set up printer for two columns
EscTxt("E");		\reset
EscTxt("(10U");		\PC-8 font with the IBM extended character set
EscTxt("(s24H");	\24 characters per inch
EscTxt("&l4C");		\VMI = 4/48" = 1/12" or 12 lines per inch
EscTxt("&l120F");	\text length = 120 lines (or bottom margin = 1/2")
EscTxt("&a10L");	\set left margin

PrintFile;

\Eject any partial sheet and reset printer to its initial configuration
EscTxt("E");
end;	\Main
